/*	$NetBSD$ */

/*-
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@nerv.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD$");

#include <sys/param.h>
#include <sys/device.h>

#include <machine/bus.h>

#include <arm/imx/imx51reg.h>
#include <arm/imx/imx51var.h>
#include <arm/imx/imx51_ipureg.h>

struct ipu_softc {
	device_t sc_axi;
	device_t sc_dev;
	bus_addr_t sc_addr;
	bus_space_tag_t sc_iot;
	bus_space_handle_t sc_ioh_framebuffer;	/* +0x00000000 - 0x0E000000 */
#define	IPU_FRAMEBUFFER_SIZE	0x0e000000
	bus_space_handle_t sc_ioh_regs1;	/* +0x0E000000 - 0x0E070000 */
#define	IPU_REGS1_SIZE		0x00070000
	bus_space_handle_t sc_ioh_regs2;	/* +0x0F000000 - 0x0E0A0000 */
#define	IPU_REGS2_SIZE		0x000a0000
	void *sc_ih_err;
	void *sc_ih_sync;
	bus_size_t sc_size;
};

static int ipu_match(device_t, struct cfdata *, void *);
static void ipu_attach(device_t, device_t, void *);
static void ipu_init(struct ipu_softc *);
static int ipu_err_intr(void *);
static int ipu_sync_intr(void *);

CFATTACH_DECL_NEW(imx51ipu, sizeof(struct ipu_softc),
    ipu_match, ipu_attach, NULL, NULL);

/* ARGSUSED */
static int
ipu_match(device_t parent __unused, struct cfdata *match __unused,
    void *aux)
{
	struct axi_attach_args *aa;

	aa = aux;

	switch (aa->aa_addr) {
	case IMX51_IPU_BASE:
		break;
	default:
		return 0;
	}

	aa->aa_size = IMX51_IPU_SIZE;

	return 1;
}

/* ARGSUSED */
static void
ipu_attach(device_t parent __unused, device_t self, void *aux)
{
	struct ipu_softc *sc;
	struct axi_attach_args *aa;

	aprint_normal(": Image Processing Unit\n");
	aprint_naive("\n");

	aa = aux;
	sc = device_private(self);
	sc->sc_axi = parent;
	sc->sc_dev = self;
	sc->sc_iot = aa->aa_iot;
	sc->sc_addr = aa->aa_addr;
	sc->sc_size = aa->aa_size;


	if (bus_space_map(sc->sc_iot, sc->sc_addr, IPU_FRAMEBUFFER_SIZE, 0,
	    &sc->sc_ioh_framebuffer))
	    panic("%s: Cannot map framebuffer", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr, IPU_REGS1_SIZE, 0,
	    &sc->sc_ioh_regs1))
	    panic("%s: Cannot map registers1", device_xname(self));
	if (bus_space_map(sc->sc_iot, sc->sc_addr, IPU_REGS2_SIZE, 0,
	    &sc->sc_ioh_regs2))
	    panic("%s: Cannot map registers2", device_xname(self));

	ipu_init(sc);

	sc->sc_ih_err = intr_establish(aa->aa_irq, IPL_CLOCK, 0,
	    ipu_err_intr, sc);
	if (sc->sc_ih_err == NULL)
		aprint_error_dev(self, "unable to register error interrupt");
	sc->sc_ih_sync = intr_establish(aa->aa_irq + 1, IPL_CLOCK, 0,
	    ipu_sync_intr, sc);
	if (sc->sc_ih_sync == NULL)
		aprint_error_dev(self, "unable to register sync interrupt");
}

static void
ipu_init(struct ipu_softc *sc)
{
	axi_gpio_pin_ctl(sc->sc_axi, 0, 0, 0);
	axi_gpio_pin_ctl(sc->sc_axi, 1, 0, 0);
	axi_gpio_pin_ctl(sc->sc_axi, 2, 0, 0);
	axi_gpio_pin_ctl(sc->sc_axi, 3, 0, 0);
}

static int
ipu_err_intr(void *arg)
{
	struct ipu_softc *sc;

	sc = (struct ipu_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}

static int
ipu_sync_intr(void *arg)
{
	struct ipu_softc *sc;

	sc = (struct ipu_softc *)arg;

	/* XXX: notyet */
	printf("%s:%d\n", __func__, __LINE__);

	return 1;
}
